﻿using System;
using System.Drawing;
using System.Windows.Forms;
using Ozeki.Media.MediaHandlers;
using Ozeki.Media.MediaHandlers.IPCamera;
using Ozeki.Media.MediaHandlers.Video;
using Ozeki.Media.Video.Controls;

namespace IPCamera_Simple
{
    public partial class IPCameraForm : Form
    {
        // IPCamera declaration.
        IPCamera _ipCamera;

        // Required for image displaying.
        DrawingImageProvider _imageProvider;
        VideoViewerWF _videoViewer;

        // Required for playing the audio.
        Speaker _speaker;

        // Required for connecting the mediahandlers.
        MediaConnector _connector;

        public IPCameraForm()
        {
            InitializeComponent();
        }

        void IPCameraForm_Load(object sender, EventArgs e)
        {
            _imageProvider = new DrawingImageProvider();
            _videoViewer = new VideoViewerWF();
            _speaker = Speaker.GetDefaultDevice();

            _connector = new MediaConnector();

            SetVideoViewer();
        }

        // Sets the videoviewer's params.
        void SetVideoViewer()
        {
            // Since the VideoViewerWF does not appear within the designer view, it
            // is being set up programmatically
            _videoViewer.Location = new Point(10, 10);
            _videoViewer.Size = new Size(500, 400);
            _videoViewer.BackColor = Color.Black;
            _videoViewer.TabStop = false;

            Controls.Add(_videoViewer);
        }

        // Initializes the camera, connects the mediahandlers.
        void InitializeCamera(string address, string userName, string password)
        {
            if (_ipCamera == null)
            {
                // Gets the camera, which can be reached by the address, and requires authentication.
                _ipCamera = IPCamera.GetCamera(address, userName, password);
                _ipCamera.CameraStateChanged += _ipCamera_CameraStateChanged;
                _ipCamera.CameraErrorOccured += _ipCamera_CameraErrorOccured;
                _speaker.Start();

                // Connects the channels to the correct receivers.
                _connector.Connect(_ipCamera.AudioChannel, _speaker);
                _connector.Connect(_ipCamera.VideoChannel, _imageProvider);

                _videoViewer.SetImageProvider(_imageProvider);
            }
        }

        // Unsubscribes from the events.
        void WireDownEvents()
        {
            if (_ipCamera == null)
                return;

            _ipCamera.CameraStateChanged -= _ipCamera_CameraStateChanged;
            _ipCamera.CameraErrorOccured -= _ipCamera_CameraErrorOccured;
        }

        // This method is being called, when the camera's state has been changed.
        void _ipCamera_CameraStateChanged(object sender, CameraState e)
        {
            InvokeGuiThread(() => { tb_Status.Text = e.Item.ToString(); });
        }

        // This method is being called, when an error occurs.
        void _ipCamera_CameraErrorOccured(object sender, CameraError e)
        {
            InvokeGuiThread(() =>
            {
                if (e.Details == null)
                {
                    tb_Error.Text = e.Error.ToString();
                }
                else
                {
                    tb_Error.Text = e.Details;
                }
            });
        }

        // Tries to connect to the device and starts to receive data from the stream.
        void btn_Start_Click(object sender, EventArgs e)
        {
            if (!String.IsNullOrEmpty(tb_CameraAddress.Text))
            {
                InitializeCamera(tb_CameraAddress.Text, tb_UserName.Text, tb_Password.Text);

                // Starts the VideoViewerWF and the IPCamera object
                _videoViewer.Start();
                _ipCamera.Start();
            }
        }

        // Disconnects from the IPCamera
        void btn_Disconnect_Click(object sender, EventArgs e)
        {
            if (_ipCamera == null)
                return;

            WireDownEvents();

            // The camera disconnects and the videoviewer stops.
            _ipCamera.Disconnect();
            _videoViewer.Stop();
        }

        // This helper method solves thread blockings.
        void InvokeGuiThread(Action action)
        {
            Invoke(action);
        }

    }
}
